--[[ 
Let's Ride 7.2.0.10

Ridez.lua is simple World of Warcraft Addon that randomly
generates a mount from the list of mounts possessed by the character.

Ridez.lua has a built in check to pick an appropriate mount or the
user can force a "special" mount or a "favorite" mount <NOT YET IMPLIMENTED>.

By default LetsRide.lua will summon a flyable mount where flying is
allowed. When flying is NOT allowed, LetsRide.lua will summon either a
ground mount or, if swimming, a water walking mount (mount must be usable by character).

Special mounts are arrays of mounts divided into mounts that have vendors, can
carry passengers (divided into flying and ground subcategories), can
walk on water, increase swim speed, and the special AQ40 only mounts.
Special mounts can be  assigned specific key assignments from the
user interface menu <NOT YET IMPLIMENTED>.


To do list:
10. Move Special Mounts Array to separate file
20. Clean up code
	a. use arrays generated from files
	b. implement ipairs
30. Winterguard
	a. Error check if current code handles PvP and non-PvP times
40. User Interface
	a. create a button
	b. create menu interface
	c. User Input
		1) allow users to "key bind" special mount arrays
		2) allow users to define mount priorities
50. Add druid travel form handling
	a. allow user to "key bind" travel form
	b. allow user to determine priority over mounts
		
*The Anglers Fishing Raft (item:85500, spell:124036) is not a mount. No
decision has been made to include / exclude the raft.
--]]

--[[
local ADDON, addon = ...
_G[ADDON] = addon
addon.frames = {}
--]]

--[[
local LRFrame = CreateFrame("FRAME")
--  LRFrame:RegisterEvent("PLAYER_LOGOUT");
  LRFrame:RegisterEvent("ADDON_LOADED");

function LRFrame:OnEvent(event, arg)
	if event == "ADDON_LOADED" and arg == "LetsRide" then
		print("Let's Ride v7.2.0.10 Loaded")
		if EFFirstRun == nil then
			EFFirstRun = true;
		end
		if EForagerDreanorNotice == nil then
			EForagerDreanorNotice = 1
		end			
	elseif event == "PLAYER_LOGOUT" then
		EFFirstRun = false;
		
		
	end
	
end
--]]

-- Simple slash command to mount
SLASH_LETSRIDE1, SLASH_LETSRIDE2 = '/lr', '/letsride';


local sMounts = {}						-- Array of Mounts with special properties

-- Passenger Ground Mounts
sMounts.gPax = {
61465,	-- Grand Black War Mammoth (Alliance)
61467,	-- Grand Black War Mammoth (Horde)
122708,	-- Grand Expedition Yak
61469,	-- Grand Ice Mammoth (Alliance)
61470,	-- Grand Ice Mammoth (Horde)
55531,	-- Mechano-Hog (Horde only)
60424,	-- Mekgineer's Chopper (Alliance only)
61425,	-- Traveler's Tundra Mammoth (Alliance)
61447	-- Traveler's Tundra Mammoth (Horde)
}

-- Passenger Flying Mounts
sMounts.fPax = {
121820,	-- Obsidian Nightwing (Heart of the Nightwing)
93326,	-- Sandstone Drake (Vial of the Sands)
75973	-- X-53 Touring Rocket
}

-- Vendor Mounts
sMounts.vendor = {
122708,	-- Grand Expedition Yak
61425,	-- Traveler's Tundra Mammoth (Alliance)
61447	-- Traveler's Tundra Mammoth (Horde)
}

-- Swimming Mounts
sMounts.swim = {
214791,	-- Brinedeep Bottom-Feeder
223018,	-- Fathom Dweller
30174,	-- Riding Turtle
64731,	-- Sea Turtle
98718	-- Subdued Seahorse
}

-- Qiraji Battle Tanks (AQ40 only)
sMounts.qiraji = {
25953,	-- Blue
26056,	-- Green
26054,	-- Red
26055	-- Yellow
}
-- Note that Black and Ultramarine are intentionally left off

-- Vashjir Abyssal Seahorse
sMounts.seahorse = {
75207	-- Abyssal (Vashj'ir) Seahorse
}

-- Water Walking Mounts
sMounts.strider = {
118089,	-- Azure Water Strider
127271	-- Crimson Water Strider
}
--

-- Check if the current mount is the special type passed
-- Takes a specialty (string) and compairs to the spellID (number) of
-- the mount in question.
-- Returns true if spellID is found.
function IsSpecialMount(specialty, spellID)
	for i,v in ipairs(sMounts[specialty]) do 
		if spellID == v then 
		 return true
		end
	end
end
--]]

-- IHaveMounts returns an array of "mount info" arrays for each 
-- mount the character can use.
function IHaveMounts()
  local myMounts, m = {}, 1 									-- An array of the mounts this character owns (can use)
--  test counters for mountTypeID
--  local totalMounts, n230, n231, n232, n241, n242, n247, n248, n254, n269, n284 = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
--  End counters
	for i,v in ipairs(C_MountJournal.GetMountIDs()) do
	  local _, _, _, _, iIsUsable, _, _, _, _, _, iIsCollected = C_MountJournal.GetMountInfoByID(v);	
		if (iIsCollected and iIsUsable) then						-- If this mount is collected and usable
		  myMounts[m] = {C_MountJournal.GetMountInfoByID(v)};			-- Add the mount information to myMounts
		  m = m+1;														-- Increment myMount
		end	
	end
  return myMounts;
 
end

-- Sort mounts into arrays of favorites and non-favorites
-- Also checks for unique mount properties like the Abyssal Seahorse or the Sandstone Drake
-- Returns sorted mount arrays
function SortMounts()
  local grnd, g, fly, f, swim, s = {}, 1, {}, 1, {}, 1		-- Create ground and flying arrays
  local fGrnd, fg, fFly, ff, fSwim, fs = {}, 1, {}, 1, {}, 1
  local gPax, gp, fPax, fp = {}, 1, {}, 1					-- Create arrays for passenger mounts
  local fGPax, fgp, fFPax, ffp = {}, 1 , {}, 1
  local vendor, ve, strider, st, qiraji, q = {}, 1, {}, 1, {}, 1	-- Create arrays for special cases
  local fVendor, fve, fStrider, fst, fQiraji, fq = {}, 1, {}, 1, {}, 1
  local seahorse, chauffeur										-- Create variable for Vashj'ir Seahorse and Heirloom Chauffeur
  local myMounts = IHaveMounts()
	
	if (myMounts[1] == nil) then return						-- If there are no usable mounts, return nil
	else
		for i,v in ipairs(myMounts) do
		  local iCreatureName, iSpellID, iIsFavorite, iMountID = v[1], v[2], v[7], v[12];
		  local _, _, _, _, iMountTypeID = C_MountJournal.GetMountInfoExtraByID(iMountID);
			if iMountTypeID == 232 then							-- If Abyssal Seahorse
			  seahorse = iMountID;									-- Save seahorse
			elseif iMountTypeID == 284 then						-- Else if Heirloom Chauffeur
			  chauffeur = iMountID;									-- Save chauffeur
						  
			elseif iIsFavorite then
				if iMountTypeID == 231 or iMountTypeID == 254 then		-- Else if a swimming mount
				  fSwim[fs] = iMountID;										-- Save swim mount
				  fs = fs+1;												-- Increment swim array index					
				
				elseif iMountTypeID == 241 then						-- Else if Quraji
				  fQiraji[fq] = iMountID;									-- Save Qaraji
				  fq = fq+1;												-- Increment Qaraji array index				
				elseif iMountTypeID == 230 or iMountTypeID == 269 then 			-- Else if ground mount
				  fGrnd[fg] = iMountID;									-- Add to ground array
				  fg = fg+1;												-- Increment ground array index
				
					if iMountTypeID == 269 then							-- If Strider
					  fStrider[fst] = iMountID;								-- Save Strider
					  fst = fst+1;											-- Increment Strider array index
					
					elseif IsSpecialMount("gPax", iSpellID) then		-- Else if ground passenger mount
					  fGPax[fgp] = iMountID;									-- Save g-passenger
					  fgp = fgp+1;											-- Increment g-passenger array index
						if IsSpecialMount("vendor", iSpellID) then			-- If vendor
						  fVendor[fve] = iMountID;								-- Save vendor
						  fve = fve+1;											-- Increment vendor array index	
						end
					end
				elseif iMountTypeID == 248 or iMountTypeID == 247 then			-- Else if is flying mount
				  fFly[ff] = iMountID;									-- Add to flying array
				  ff = ff+1;												-- Increment flying array index
					if IsSpecialMount("fPax", iSpellID) then			-- If flying passenger mount
					  fFPax[ffp] = iMountID;									-- Save f-passenger
					  ffp = ffp+1;											-- Increment f-passenger array index
					end
				else												-- Else we didn't identify the mount
				  print("Error, mountTypeID undefined!")				-- Report finding
				  print("Mount name = " .. iCreatureName .. ", mountID=" .. iMountID .. ", mountTypeID=" .. iMountTypeID);
				end	
						
			elseif not iIsFavorite then
				if iMountTypeID == 231 or iMountTypeID == 254 then		-- Else if a swimming mount
				  swim[s] = iMountID;										-- Save swim mount
				  s = s+1;													-- Increment swim array index					
				
				elseif iMountTypeID == 241 then						-- Else if Quraji
				  qiraji[q] = iMountID;									-- Save Qaraji
				  q = q+1;												-- Increment Qaraji array index				
				elseif iMountTypeID == 230 or iMountTypeID == 269 then 			-- Else if ground mount
				  grnd[g] = iMountID;									-- Add to ground array
				  g = g+1;												-- Increment ground array index
				
					if iMountTypeID == 269 then							-- If Strider
					  strider[st] = iMountID;								-- Save Strider
					  st = st+1;											-- Increment Strider array index
					
					elseif IsSpecialMount("gPax", iSpellID) then		-- Else if ground passenger mount
					  gPax[gp] = iMountID;									-- Save g-passenger
					  gp = gp+1;											-- Increment g-passenger array index
						if IsSpecialMount("vendor", iSpellID) then			-- If vendor
						  vendor[ve] = iMountID;								-- Save vendor
						  ve = ve+1;											-- Increment vendor array index
						end
					end
				elseif iMountTypeID == 248 or iMountTypeID == 247 then			-- Else if is flying mount
				  fly[f] = iMountID;									-- Add to flying array
				  f = f+1;												-- Increment flying array index
					if IsSpecialMount("fPax", iSpellID) then			-- If flying passenger mount
					  fPax[fp] = iMountID;									-- Save f-passenger
					  fp = fp+1;											-- Increment f-passenger array index
					end
				else												-- Else we didn't identify the mount
				  print("Error, mountTypeID undefined!")				-- Report finding
				  print("Mount name = " .. iCreatureName .. ", mountID=" .. iMountID .. ", mountTypeID=" .. iMountTypeID);
				end
			else												-- Else somehow is not and is not-not a favorite...
			  print("Eureka! Now entering quantim mounting!")		-- Report finding!
			end
		end
	end
	
	if fg == 1 then fGrnd = grnd; print("You had no favorite ground mounts")
	elseif g == 1 then grnd = fGrnd; print("You had no non-favorite ground mounts")
	end
	
	if fgp == 1 then fGPax = gPax;
	elseif gp == 1 then gPax = fGPax;
	end
	
	if ff == 1 then fFly = fly; print("You had no favorite flying mounts")
	elseif f == 1 then fly = fFly; print("You had no non-favorite ground mounts")
	end
	
	if ffp == 1 then fFPax = fPax;
	elseif fp == 1 then fPax = fFPax;
	end
	
	if fve == 1 then fVendor = vendor;
	elseif ve == 1 then vendor = fVendor;
	end
	
	if fs == 1 then fSwim = swim;
	elseif s == 1 then swim = fSwim;
	end
	
	if fq == 1 then fQiraji = qiraji;
	elseif q == 1 then qiraji = fQiraji;
	end
	
	if fst == 1 then fStrider = strider;
	elseif st == 1 then strider = fStrider;
	end
	
  local sortedMounts = {grnd, gPax, fly, fPax, vendor, swim, qiraji, seahorse, strider, chauffeur}
  local favoriteMounts = {fGrnd, fGPax, fFly, fFPax, fVendor, fSwim, fQiraji, seahorse, fStrider, chauffeur}
  return sortedMounts, favoriteMounts

end

-- MountUp determines which mount should be called
-- If appropriate, will use random to decide on the mount
function MountUp(btn, myMounts)
  -- Currently ALT, Shift and right-click are hard coded. Need to write code
  -- that allows the user to determine keys for special cases
  local grnd, gPax, fly, fPax, vendor = myMounts[1], myMounts[2], myMounts[3], myMounts[4], myMounts[5]
  local swim, qiraji, seahorse, strider, chauffeur = myMounts[6], myMounts[7], myMounts[8], myMounts[9], myMounts[10]
  local zone = "blank"
	if (GetMapInfo() ~= nil) then
	  zone = GetMapInfo();
	end
	
	-- User overrides
	if (IsAltKeyDown() and btn == "RightButton" and swim) then	--Force swimming mounts
	  C_MountJournal.SummonByID( swim[random(#swim)]);
	elseif (IsAltKeyDown() and btn == "LeftButton" and strider) then
	  C_MountJournal.SummonByID( strider[random(#strider)]);        -- Force Water Strider
	elseif (IsShiftKeyDown() and btn == "LeftButton" and IsFlyableArea() and fPax) then 
	  C_MountJournal.SummonByID( fPax[random(#fPax)]);				-- Force passenger mounts
	elseif (IsShiftKeyDown() and btn == "LeftButton" and gPax) then
	  C_MountJournal.SummonByID( gPax[random(#gPax)]);
	elseif (IsShiftKeyDown() and btn == "RightButton" and gPax) then
	  C_MountJournal.SummonByID( gPax[random(#gPax)]);
	elseif (btn == "RightButton" and vendor) then				--Force vendors
	  C_MountJournal.SummonByID( vendor[random(#vendor)]);
	  
	-- Special Zones
	-- The Battle Tanks can only be summoned in Temple of Ahn'Qiraj (AQ40) so the zone
	-- must match exactly
	-- The Vashj'ir seahorse can be mounted in any of the Vashj'ir zones so the string.match
	-- function is called
	elseif (zone == "AhnQiraj" and qiraji) then					-- If in AQ40
	  C_MountJournal.SummonByID( qiraji[random(#qiraji)]);			-- Attempt to mount the battle tanks
	elseif (zone:match("Vashjir") and IsSwimming() and seahorse) then
	  C_MountJournal.SummonByID( seahorse);							-- Attempt to mount the Vashj'ir seahorse
	
	-- Best choice algorith picks a random mount from the appropriate category 
	elseif (IsFlyableArea() and fly) then				-- If flying is allowed
	  C_MountJournal.SummonByID( fly[random(#fly)]);		-- Call random flying mount
	elseif (IsSwimming() and strider) then				-- If cannot fly while swimming
	  C_MountJournal.SummonByID( strider[random(#strider)]);	-- Attempt to call the Strider
	elseif (IsSwimming() and swim) then					-- If no Strider while swimming
	  C_MountJournal.SummonByID( swim[random(#swim)]);		-- Attempt to call a swimming mount
	elseif grnd then
	  C_MountJournal.SummonByID( grnd[random(#grnd)]);
	elseif chauffeur then
	  C_MountJournal.SummonByID( chauffeur);
	else print("Well, you broke it. Contact Ridez administrator for tech support.")
	end
end

function SlashCmdList.LETSRIDE(msg, editbox)
--[[  local button = CreateFrame("Button", nil, mainframe)
	button:SetPoint("CENTER", mainframe, "CENTER", 0, 0)
	button:SetWidth(200)
	button:SetHeight(50)
	
	button:SetText("test")
	button:SetNormalFontObject("GameFontNormal")
	
  local ntex = button:CreateTexture()
	ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
	ntex:SetTexCoord(0, 0.625, 0, 0.6875)
	ntex:SetAllPoints()	
	button:SetNormalTexture(ntex)
	
  local htex = button:CreateTexture()
	htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
	htex:SetTexCoord(0, 0.625, 0, 0.6875)
	htex:SetAllPoints()
	button:SetHighlightTexture(htex)
	
  local ptex = button:CreateTexture()
	ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
	ptex:SetTexCoord(0, 0.625, 0, 0.6875)
	ptex:SetAllPoints()
	button:SetPushedTexture(ptex)
--]]  
  local btn = GetMouseButtonClicked();
  local favPriority = 10									-- temp value, will eventually be replaced by user input (1-11)
	
	if IsMounted() then 									--If you are mounted
		if (btn=="LeftButton" or btn=="RightButton") then		-- Check if mouse button 1 or 2 was pressed
		  EjectPassengerFromSeat(btn=="LeftButton" and 1 or 2);		-- Eject respective passenger
		elseif (btn=="MiddleButton") then						-- Else if middle mouse button was pressed
		  C_MountJournal.Dismiss();									-- Dismount (dismiss Active mount)
		end
	
	elseif InCombatLockdown() then					 			-- If not permitted to mount
	  print("Can't mount while in combat")							-- Warn the user
	
	elseif (not IsMounted()) then								-- If not mounted
		if (btn=="MiddleButton") then								-- If using the raft
		  print("Not yet implemented")									-- Currently do nothing
		  -- C_ToyBox.GetToyInfo(85500)									-- 85500 is the Anglers Raft
		else
-- Testing
		--addon:GetSpecialMounts();
-- End Testing

		
		  local sortedMounts, favoriteMounts = SortMounts();		-- Sort mounts into appropriate arrays
			if sortedMounts == nil then									-- If you have no mounts that you can ride
			  print("You have no mounts you can use in this area");			-- Warn the user
			else
				local favor = random(10)
				if favor < favPriority then
				  MountUp(btn, favoriteMounts);								-- "It means, 'Get on your horses.'"
				else
				  MountUp(btn, sortedMounts);
				end
			end
--]]			
		end
	
	end
	
end